@@ -29,6 +29,7 @@ class Pandora:
         base_url_rid = base_url + "rid=%sP&method=%s"
         base_url_lid = base_url + "rid=%sP&lid=%s&method=%s"
         proxy = ""
+	offset = 0
 
         def __init__( self ):
                 self.rid = "%07i" %( time.time() % 10000000 )
@@ -46,6 +47,9 @@ class Pandora:
 			raise PandoraError("Unable to load Keys");
                 self.curFormat = format
 
+	def _timestamp( self ):
+		return _inttime() - self.offset
+
         def set_proxy( self, proxy ):
                 if len(proxy)>0:
                         proxy_handler = ProxyHandler({"http": proxy})
@@ -55,24 +59,24 @@ class Pandora:
                 install_opener(opener)
 
         def sync( self ):
-                reqUrl = self.base_url_rid %( self.rid, "sync" )
+		reqUrl = self.base_url_rid %( self.rid, "sync" )
 
-                req = xmlrpclib.dumps( (), "misc.sync" ).replace( "\n", "" )
-                enc = crypt.encryptString( req )
+		req = xmlrpclib.dumps( (), "misc.sync" ).replace( "\n", "" )
+		enc = crypt.encryptString( req, self.keys['out'] )
 
-                try:
-                        u = urlopen( reqUrl, enc )
-                        resp = u.read()
-                        u.close()
-                except:
-                        return False
+		u = urlopen( reqUrl, enc )
+		resp = u.read()
+		u.close()
+
+		parsed = xmlrpclib.loads( resp )[0][0]
+		t = crypt.decryptString( parsed, self.keys['in'] )
+		self.offset = _inttime() - int(t[4:-2])
 
         def authListener( self, user, pwd ):
                 new_version = self.protocol_version
                 while True:
                         reqUrl = self.base_url_rid %( self.rid, "authenticateListener" )
-                        req = xmlrpclib.dumps( ( _inttime(), user, pwd ), \
-                                                "listener.authenticateListener" )
+                        req = xmlrpclib.dumps( ( self._timestamp(), user, pwd, 'html5tuner', '', '', 'HTML5', True ), "listener.authenticateListener" )
                         req = req.replace( "\n", "" )
                         enc = crypt.encryptString( req, self.keys['out'] )
                         try:
@@ -106,7 +110,7 @@ class Pandora:
         def getStations( self ):
                 reqUrl = self.base_url_lid %( self.rid, self.lid, "getStations" )
 
-                req = xmlrpclib.dumps( ( _inttime(), self.authToken ), \
+                req = xmlrpclib.dumps( ( self._timestamp(), self.authToken ), \
                                                                 "station.getStations" )
                 req = req.replace( "\n", "" )
                 enc = crypt.encryptString( req, self.keys['out'] )
@@ -134,7 +138,7 @@ class Pandora:
 
                 reqUrl = self.base_url_lid %( self.rid, self.lid, "addFeedback" )
 
-                args = ( _inttime(), self.authToken, stationId, song[1]["musicId"], \
+                args = ( self._timestamp(), self.authToken, stationId, song[1]["musicId"], \
                         song[1]["matchingSeed"], song[1]["userSeed"], song[1]["focusTraitId"], '', rating, False )
 
                 req = xmlrpclib.dumps( args, "station.addFeedback" )
@@ -162,7 +166,7 @@ class Pandora:
 
                 reqUrl = self.base_url_lid %( self.rid, self.lid, "addSeed" )
 
-                args = ( _inttime(), self.authToken, stationId, musicId )
+                args = ( self._timestamp(), self.authToken, stationId, musicId )
 
                 req = xmlrpclib.dumps( args, "station.addSeed" )
 
@@ -188,7 +192,7 @@ class Pandora:
 
                 reqUrl = self.base_url_lid %( self.rid, self.lid, "addTiredSong" )
 
-                args = ( _inttime(), self.authToken, identity )
+                args = ( self._timestamp(), self.authToken, identity )
 
                 req = xmlrpclib.dumps( args, "listener.addTiredSong" )
                 req = req.replace( "\n", "" )
@@ -213,7 +217,7 @@ class Pandora:
 
                 reqUrl = self.base_url_lid %( self.rid, self.lid, "search" )
 
-                args = ( _inttime(), self.authToken, search )
+                args = ( self._timestamp(), self.authToken, search )
 
                 req = xmlrpclib.dumps( args, "music.search" )
                 req = req.replace( "\n", "" )
@@ -238,7 +242,7 @@ class Pandora:
 
                 reqUrl = self.base_url_lid %( self.rid, self.lid, "createStation" )
 
-                args = ( _inttime(), self.authToken, "mi"+musicId, "" )
+                args = ( self._timestamp(), self.authToken, "mi"+musicId, "" )
 
                 req = xmlrpclib.dumps( args, "station.createStation" )
                 req = req.replace( "\n", "" )
@@ -263,7 +267,7 @@ class Pandora:
 
                 reqUrl = self.base_url_lid %( self.rid, self.lid, "removeStation" )
 
-                args = ( _inttime(), self.authToken, stationId )
+                args = ( self._timestamp(), self.authToken, stationId )
 
                 req = xmlrpclib.dumps( args, "station.removeStation" )
                 req = req.replace( "\n", "" )
@@ -289,7 +293,7 @@ class Pandora:
                         format = self.curFormat
                 reqUrl = self.base_url_lid %( self.rid, self.lid, "getFragment" )
 
-                args = ( _inttime(), self.authToken, stationId, "0", "", "", \
+                args = ( self._timestamp(), self.authToken, stationId, "0", "", "", \
                                         format, "0", "0" )
                 req = xmlrpclib.dumps( args, "playlist.getFragment" )
                 req = req.replace( "\n", "" )
