@@ -30,7 +30,7 @@ class RadioPlayer(QtCore.QThread): #Main radio class
 		self.format = 'aacplus'
 		self.protocol_version = 0
 		self.proxy = ""
-		self.pandora = 0
+		self.pandora = None
 
 		QtCore.QThread.__init__(self)
 
@@ -84,6 +84,7 @@ class RadioPlayer(QtCore.QThread): #Main radio class
 		return ret
 
 	def auth( self, user, password ): #Authenticate to Pandora
+		self.pandora.sync()
 		ret = self.pandora.authListener( str(user), str(password) )
 		return ret
 
